@@ -1,18 +1,29 @@
 #!/bin/sh
-# /etc/init.d/hwconf.sh - SliTaz hardware autoconfiguration.
+# /etc/init.d/hwconf.sh - SliTaz hardware configuration.
+#
+# This script configure the sound card and screen. Tazhw is used earlier
+# at boot time to autoconfigure PCI and USB devices.
 #
 . /etc/init.d/rc.functions
 
-# Detect PCI and USB devices with Tazhw from slitaz-tools. We load
-# kernel module only at first boot or in LiveCD mode.
-if [ ! -s /var/lib/detected-modules ]; then
-	/sbin/tazhw init
-fi
+# Parse cmdline args for boot options (See also rcS and bootopts.sh).
+for opt in `cat /proc/cmdline`
+do
+	case $opt in
+		sound=*)
+			DRIVER=${opt#sound=} ;;
+		xargs=*)
+			XARGS=${opt#xargs=} ;;
+		screen=*)
+			SCREEN=${opt#screen=} ;;
+		*)
+			continue ;;
+	esac
+done
 
 # Sound configuration stuff. First check if sound=no and remove all
 # sound Kernel modules.
-if grep -q "sound=" /proc/cmdline; then
-	DRIVER=`cat /proc/cmdline | sed 's/.*sound=\([^ ]*\).*/\1/'`
+if [ -n "$DRIVER" ]; then
 	case "$DRIVER" in
 	no)
 		echo -n "Removing all sound kernel modules..."
@@ -26,14 +37,14 @@ if grep -q "sound=" /proc/cmdline; then
 		for i in alsa-lib mhwaveedit asunder libcddb ; do
 			echo 'y' | tazpkg remove $i > /dev/null
 		done
-		status;;
+		status ;;
 	noconf)
-		echo "Sound configuration was disabled from cmdline...";;
+		echo "Sound configuration was disabled from cmdline..." ;;
 	*)
 		if [ -x /usr/sbin/soundconf ]; then
 			echo "Using sound kernel module $DRIVER..."
 			/usr/sbin/soundconf -M $DRIVER
-		fi;;
+		fi ;;
 	esac
 # Sound card may already be detected by PCI-detect.
 elif [ -d /proc/asound ]; then
@@ -56,20 +67,30 @@ fi
 if [ ! -s /etc/X11/screen.conf -a -x /usr/bin/slim ]; then
 	# $HOME is not yet set.
 	HOME=/root
-	if grep -q "xarg=*" /proc/cmdline; then
+	if [ -n "$XARGS" ]; then
 		# Add an extra argument to xserver_arguments (xarg=-2button)
-		XARG=`cat /proc/cmdline | sed 's/.*xarg=\([^ ]*\).*/\1/'`
 		sed -i "s|-screen|$XARG -screen|" /etc/slim.conf
 	fi
-	if grep -q "screen=*" /proc/cmdline; then
-		export NEW_SCREEN=`cat /proc/cmdline | sed 's/.*screen=\([^ ]*\).*/\1/'`
-		if [ "$NEW_SCREEN" = "text" ]; then
-			echo -n "Disabling X login manager: slim..."
-			sed -i s/'slim'/''/ /etc/rcS.conf
-			status
-		else
-			tazx `cat /etc/X11/wm.default`
-		fi
+	if [ -n "$SCREEN" ]; then
+		case "$SCREEN" in
+			text)
+				# Disable X.
+				echo -n "Disabling X login manager: slim..."
+				. /etc/rcS.conf
+				RUN_DAEMONS=`echo $RUN_DAEMONS | sed s/' slim'/''/`
+				sed -i s/"RUN_DAEMONS.*"/"RUN_DAEMONS=\"$RUN_DAEMONS\"/" /etc/rcS.conf
+				status ;;
+			auto)
+				# Auto detect screen resolution.
+				export NEW_SCREEN=`Xvesa -listmodes 2>&1 | grep ^0x | \
+					awk '{ printf "%s %s\n",$2 }' \
+					| sort -nr | grep x[1-2][4-6] | head -n 1`
+				tazx `cat /etc/X11/wm.default` ;;
+			*)
+				# Use specified screen resolution.
+				export NEW_SCREEN=$SCREEN
+				tazx `cat /etc/X11/wm.default` ;;
+		esac
 	else
 		tazx `cat /etc/X11/wm.default`
 	fi
